@@ -24,7 +24,7 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
-#include <config.h>
+#include "config.h"
 
 #include "gdkscreen.h"
 #include "gdkdisplay.h"
@@ -87,12 +87,12 @@ gdk_display_manager_class_init (GdkDisplayManagerClass *klass)
    * Since: 2.2
    */
   signals[DISPLAY_OPENED] =
-    g_signal_new (g_intern_static_string ("display_opened"),
+    g_signal_new (g_intern_static_string ("display-opened"),
 		  G_OBJECT_CLASS_TYPE (object_class),
 		  G_SIGNAL_RUN_LAST,
 		  G_STRUCT_OFFSET (GdkDisplayManagerClass, display_opened),
 		  NULL, NULL,
-		  gdk_marshal_VOID__OBJECT,
+		  _gdk_marshal_VOID__OBJECT,
 		  G_TYPE_NONE,
 		  1,
 		  GDK_TYPE_DISPLAY);
@@ -147,11 +147,12 @@ gdk_display_manager_get_property (GObject      *object,
     }
 }
 
-/** 
+/**
  * gdk_display_manager_get:
- * @returns: the singleton #GdkDisplayManager object.
  *
- * Returns the global #GdkDisplayManager singleton; gdk_parse_pargs(),
+ * Gets the singleton #GdkDisplayManager object.
+ *
+ * Returns: (transfer none): The global #GdkDisplayManager singleton; gdk_parse_pargs(),
  * gdk_init(), or gdk_init_check() must have been called first.
  *
  * Since: 2.2
@@ -171,9 +172,9 @@ gdk_display_manager_get (void)
  * gdk_display_manager_get_default_display:
  * @display_manager: a #GdkDisplayManager 
  *
- * Gets the default #GdkDisplay. 
- * 
- * Returns: a #GdkDisplay, or %NULL if there is no default
+ * Gets the default #GdkDisplay.
+ *
+ * Returns: (transfer none): a #GdkDisplay, or %NULL if there is no default
  *   display.
  *
  * Since: 2.2
@@ -188,12 +189,10 @@ gdk_display_manager_get_default_display (GdkDisplayManager *display_manager)
  * gdk_display_get_default:
  *
  * Gets the default #GdkDisplay. This is a convenience
- * function for:
- * <programlisting>
- *   gdk_display_manager_get_default_display (gdk_display_manager_get ())
- * </programlisting>
- * 
- * Returns: a #GdkDisplay, or %NULL if there is no default
+ * function for
+ * <literal>gdk_display_manager_get_default_display (gdk_display_manager_get ())</literal>.
+ *
+ * Returns: (transfer none): a #GdkDisplay, or %NULL if there is no default
  *   display.
  *
  * Since: 2.2
@@ -209,8 +208,8 @@ gdk_display_get_default (void)
  *
  * Gets the default screen for the default display. (See
  * gdk_display_get_default ()).
- * 
- * Returns: a #GdkScreen, or %NULL if there is no default display.
+ *
+ * Returns: (transfer none): a #GdkScreen, or %NULL if there is no default display.
  *
  * Since: 2.2
  */
