@@ -24,7 +24,7 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
-#include <config.h>
+#include "config.h"
 #include <string.h>
 #include "gtkframe.h"
 #include "gtklabel.h"
@@ -306,8 +306,8 @@ gtk_frame_forall (GtkContainer *container,
 /**
  * gtk_frame_set_label:
  * @frame: a #GtkFrame
- * @label: the text to use as the label of the frame
- * 
+ * @label: (allow-none): the text to use as the label of the frame
+ *
  * Sets the text of the label. If @label is %NULL,
  * the current label is removed.
  **/
@@ -592,7 +592,7 @@ gtk_frame_expose (GtkWidget      *widget,
     {
       gtk_frame_paint (widget, &event->area);
 
-      (* GTK_WIDGET_CLASS (gtk_frame_parent_class)->expose_event) (widget, event);
+      GTK_WIDGET_CLASS (gtk_frame_parent_class)->expose_event (widget, event);
     }
 
   return FALSE;
