@@ -20,6 +20,9 @@
 #include "config.h"
 
 #include <string.h>
+
+#undef GTK_DISABLE_DEPRECATED
+
 #include <gtk/gtk.h>
 #ifdef GDK_WINDOWING_X11
 #include <gdk/x11/gdkx.h>
@@ -103,6 +106,7 @@ static void       gail_widget_real_initialize    (AtkObject     *obj,
                                                   gpointer      data);
 static GtkWidget* gail_widget_find_viewport      (GtkWidget     *widget);
 static gboolean   gail_widget_on_screen          (GtkWidget     *widget);
+static gboolean   gail_widget_all_parents_visible(GtkWidget     *widget);
 
 G_DEFINE_TYPE_WITH_CODE (GailWidget, gail_widget, GTK_TYPE_ACCESSIBLE,
                          G_IMPLEMENT_INTERFACE (ATK_TYPE_COMPONENT, atk_component_interface_init))
@@ -239,7 +243,6 @@ gail_widget_get_description (AtkObject *accessible)
     {
       /* Get the tooltip from the widget */
       GtkAccessible *obj = GTK_ACCESSIBLE (accessible);
-      GtkTooltipsData *data;
 
       gail_return_val_if_fail (obj, NULL);
 
@@ -250,12 +253,8 @@ gail_widget_get_description (AtkObject *accessible)
         return NULL;
  
       gail_return_val_if_fail (GTK_WIDGET (obj->widget), NULL);
-    
-      data = gtk_tooltips_data_get (obj->widget);
-      if (data == NULL)
-        return NULL;
 
-      return data->tip_text;
+      return gtk_widget_get_tooltip_text (obj->widget);
     }
 }
 
@@ -515,8 +514,8 @@ gail_widget_ref_state_set (AtkObject *accessible)
       if (GTK_WIDGET_VISIBLE (widget))
         {
           atk_state_set_add_state (state_set, ATK_STATE_VISIBLE);
-          if (gail_widget_on_screen (widget) &&
-              GTK_WIDGET_MAPPED (widget))
+          if (gail_widget_on_screen (widget) && GTK_WIDGET_MAPPED (widget) &&
+              gail_widget_all_parents_visible (widget))
             {
               atk_state_set_add_state (state_set, ATK_STATE_SHOWING);
             }
@@ -1076,3 +1075,30 @@ static gboolean gail_widget_on_screen (GtkWidget *widget)
 
   return return_value;
 }
+
+/**
+ * gail_widget_all_parents_visible:
+ * @widget: a #GtkWidget
+ *
+ * Checks if all the predecesors (the parent widget, his parent, etc) are visible
+ * Used to check properly the SHOWING state.
+ *
+ * Return value: TRUE if all the parent hierarchy is visible, FALSE otherwise
+ **/
+static gboolean gail_widget_all_parents_visible (GtkWidget *widget)
+{
+  GtkWidget *iter_parent = NULL;
+  gboolean result = TRUE;
+
+  for (iter_parent = gtk_widget_get_parent (widget); iter_parent;
+       iter_parent = gtk_widget_get_parent (iter_parent))
+    {
+      if (!GTK_WIDGET_VISIBLE (iter_parent))
+        {
+          result = FALSE;
+          break;
+        }
+    }
+
+  return result;
+}
